@@ -74,7 +74,7 @@ let close e =
   e.count<- return_no_more_count;
   e.clone<- empty
 
-let force t =(** Transform [t] into a list *)
+let force t = (* Transform [t] into a list *)
   let rec clone enum count =
     let enum = ref !enum
     and count = ref !count in
@@ -265,7 +265,7 @@ let take n e =
   let r = ref [] in
   begin
     try
-      for i = 1 to n do
+      for _i = 1 to n do
         r := e.next () :: !r
       done
     with No_more_elements -> ()
@@ -740,7 +740,7 @@ let range ?until x =
 *)
 
 let drop n e =
-  for i = 1 to n do
+  for _i = 1 to n do
     junk e
   done
 
@@ -987,7 +987,7 @@ let clump clump_size add get e = (* convert a uchar enum into a ustring enum *)
       add x;
       junk e; (* don't get [x] twice *)
       (try
-        for i = 2 to clump_size do
+        for _i = 2 to clump_size do
           add (e.next ())
         done
       with No_more_elements -> ());
@@ -1597,47 +1597,3 @@ end
     let id l = Monad.bind l Monad.return in  \
     List.enum l |> id |> List.of_enum = l)
 *)
-
-module Incubator = struct
-  open BatOrd
-
-  let int_eq (x:int) y = x = y
-  let int_ord (x:int) y =
-    if x > y then Gt
-    else if y > x then Lt
-    else Eq
-
-  let eq_elements eq_elt a1 a2 = for_all2 eq_elt a1 a2
-
-  let rec ord_elements ord_elt t u =
-    match (get t, get u) with
-    | (None, None)     -> Eq
-    | (None, _)        -> Lt
-    | (_, None)        -> Gt
-    | (Some x, Some y) -> match ord_elt x y with
-      | Eq -> ord_elements ord_elt t u
-      | (Gt|Lt) as n -> n
-
-  let eq eq_elt t1 t2 =
-    bin_eq
-      int_eq (count t1) (count t2)
-      (eq_elements eq_elt) t1 t2
-
-  let ord ord_elt t1 t2 =
-    bin_ord
-      int_ord (count t1) (count t2)
-      (ord_elements ord_elt) t1 t2
-
-  module Eq (T : Eq) = struct
-    type 'a enum = 'a t
-    type t = T.t enum
-    let eq = eq T.eq
-  end
-
-  module Ord (T : Ord) = struct
-    type 'a enum = 'a t
-    type t = T.t enum
-    let ord = ord T.ord
-  end
-
-end
