@@ -31,8 +31,7 @@ let count_lines (fn: string): int =
   !count
 
 (*$T count_lines
-  (Sys.file_exists "/dev/null") && (count_lines "/dev/null" = 0)
-  (Sys.file_exists "/etc/services") && (count_lines "/etc/services" > 0)
+  (Sys.file_exists __FILE__) && (count_lines __FILE__ > 0)
 *)
 
 open BatIO
@@ -146,7 +145,7 @@ let open_in ?mode ?(perm=default_permission) name =
       ~read:(fun () ->
         if !pos >= len then raise No_more_input
         else Array1.get array (BatRef.post_incr pos))
-      ~input:(fun sout p l ->
+      ~input:(fun sout _p l ->
         if !pos >= len then raise No_more_input;
         let n = (if !pos + l > len then len - !pos else l) in
         for i = 0 to n - 1 do
