@@ -34,8 +34,6 @@
     @author David Teller
 *)
 
-open Hashtbl
-
 type ('a, 'b) t = ('a, 'b) Hashtbl.t
 (** A Hashtable with keys of type 'a and values 'b *)
 
@@ -273,20 +271,6 @@ val hash : 'a -> int
     Moreover, [hash] always terminates, even on cyclic
     structures. *)
 
-external hash_param : int -> int -> 'a -> int = "caml_hash_univ_param" "noalloc"
-(** [Hashtbl.hash_param n m x] computes a hash value for [x], with the
-    same properties as for [hash]. The two extra parameters [n] and
-    [m] give more precise control over hashing. Hashing performs a
-    depth-first, right-to-left traversal of the structure [x], stopping
-    after [n] meaningful nodes were encountered, or [m] nodes,
-    meaningful or not, were encountered. Meaningful nodes are: integers;
-    floating-point numbers; strings; characters; booleans; and constant
-    constructors. Larger values of [m] and [n] means that more
-    nodes are taken into account to compute the final hash
-    value, and therefore collisions are less likely to happen.
-    However, hashing takes longer. The parameters [m] and [n]
-    govern the tradeoff between accuracy and speed. *)
-
 
 (** {6 Boilerplate code}*)
 
@@ -371,8 +355,10 @@ module type HashedType =
 sig
   type t
   (** The type of the hashtable keys. *)
+
   val equal : t -> t -> bool
   (** The equality predicate used to compare keys. *)
+
   val hash : t -> int
     (** A hashing function on keys. It must be such that if two keys are
         equal according to [equal], then they have identical hash values
