@@ -30,6 +30,17 @@ module Concrete = struct
   let empty = Empty
 
   let is_empty = function Empty -> true | _ -> false
+
+  let is_singleton = function
+    | Node (Empty, _x, Empty, _h) -> true
+    | _ -> false
+
+  (*$T is_singleton
+    is_singleton (of_list []) = false
+    is_singleton (of_list [1]) = true
+    is_singleton (of_list [1;2]) = false
+  *)
+
   (* Sets are represented by balanced binary trees (the heights of the
      children differ by at most 2 *)
   let height = function
@@ -82,8 +93,13 @@ module Concrete = struct
   (* Smallest and greatest element of a set *)
   let rec min_elt = function
       Empty -> raise Not_found
-    | Node(Empty, v, r, _) -> v
-    | Node(l, v, r, _) -> min_elt l
+    | Node(Empty, v, _r, _) -> v
+    | Node(l, _v, _r, _) -> min_elt l
+
+  let rec min_elt_opt = function
+      Empty -> None
+    | Node(Empty, v, _r, _) -> Some v
+    | Node(l, _v, _r, _) -> min_elt_opt l
 
   let get_root = function
     | Empty -> raise Not_found
@@ -111,13 +127,18 @@ module Concrete = struct
 
   let rec max_elt = function
       Empty -> raise Not_found
-    | Node(l, v, Empty, _) -> v
-    | Node(l, v, r, _) -> max_elt r
+    | Node(_l, v, Empty, _) -> v
+    | Node(_l, _v, r, _) -> max_elt r
+
+  let rec max_elt_opt = function
+      Empty -> None
+    | Node(_l, v, Empty, _) -> Some v
+    | Node(_l, _v, r, _) -> max_elt_opt r
 
   (* Remove the smallest element of the given set *)
   let rec remove_min_elt = function
       Empty -> invalid_arg "Set.remove_min_elt"
-    | Node(Empty, v, r, _) -> r
+    | Node(Empty, _v, r, _) -> r
     | Node(l, v, r, _) -> bal (remove_min_elt l) v r
 
   (* Merge two trees l and r into one.
@@ -137,18 +158,42 @@ module Concrete = struct
 
   (* Insertion of one element *)
   let rec add cmp x = function
-      Empty -> Node(Empty, x, Empty, 1)
+    | Empty -> Node(Empty, x, Empty, 1)
     | Node(l, v, r, _) as t ->
       let c = cmp x v in
-      if c = 0 then t else
-      if c < 0 then bal (add cmp x l) v r else bal l v (add cmp x r)
+      if c = 0 then
+        t
+      else if c < 0 then
+        let nl = add cmp x l in
+        if nl == l then
+          t
+        else
+          bal nl v r
+      else
+        let nr = add cmp x r in
+        if nr == r then
+          t
+        else
+          bal l v nr
 
   let rec remove cmp x = function
-      Empty -> Empty
-    | Node(l, v, r, _) ->
+    | Empty as t -> t
+    | Node(l, v, r, _) as t ->
       let c = cmp x v in
-      if c = 0 then merge l r else
-      if c < 0 then bal (remove cmp x l) v r else bal l v (remove cmp x r)
+      if c = 0 then
+        merge l r
+      else if c < 0 then
+        let nl = remove cmp x l in
+        if nl == l then
+          t
+        else
+          bal nl v r
+      else
+        let nr = remove cmp x r in
+        if nr == r then
+          t
+        else
+          bal l v nr
 
   (* A variant of [remove] that throws [Not_found] on failure *)
   let rec remove_exn cmp x = function
@@ -169,14 +214,25 @@ module Concrete = struct
     else
       let rec loop = function
         | Empty -> raise Not_found
-        | Node(l, v, r, h) ->
-          let c = cmp x v in
-          if c = 0 then
-            Node(l, y, r, h)
-          else if c < 0 then
-            Node(loop l, v, r, h)
-          else
-            Node(l, v, loop r, h)
+        | Node(l, v, r, h) as t ->
+           let c = cmp x v in
+           if c = 0 then
+             if v == y then
+               t
+             else
+               Node(l, y, r, h)
+           else if c < 0 then
+             let nl = loop l in
+             if nl == l then
+               t
+             else
+               Node(nl, v, r, h)
+           else
+             let nr = loop r in
+             if nr == r then
+               t
+             else
+               Node(l, v, nr, h)
       in
       loop s
 
@@ -192,6 +248,58 @@ module Concrete = struct
       let c = cmp x v in
       if c = 0 then v else  find cmp x (if c < 0 then l else r)
 
+  let rec find_opt cmp x = function
+      Empty -> None
+    | Node(l, v, r, _) ->
+      let c = cmp x v in
+      if c = 0 then Some v else find_opt cmp x (if c < 0 then l else r)
+
+  let rec find_first_helper_found k0 f = function
+    | Empty -> k0
+    | Node (l, k, r, _) ->
+       if f k
+       then find_first_helper_found k f l
+       else find_first_helper_found k0 f r
+
+  let rec find_first f m =
+    match m with
+    | Empty -> raise Not_found
+    | Node (l, k, r, _) ->
+       if f k
+       then find_first_helper_found k f l
+       else find_first f r
+
+  let rec find_first_opt f m =
+    match m with
+    | Empty -> None
+    | Node (l, k, r, _) ->
+       if f k
+       then Some (find_first_helper_found k f l)
+       else find_first_opt f r
+
+  let rec find_last_helper_found k0 f = function
+    | Empty -> k0
+    | Node (l, k, r, _) ->
+       if f k
+       then find_last_helper_found k f r
+       else find_last_helper_found k0 f l
+
+  let rec find_last f m =
+    match m with
+    | Empty -> raise Not_found
+    | Node (l, k, r, _) ->
+       if f k
+       then find_last_helper_found k f r
+       else find_last f l
+
+  let rec find_last_opt f m =
+    match m with
+    | Empty -> None
+    | Node (l, k, r, _) ->
+       if f k
+       then Some (find_last_helper_found k f r)
+       else find_last_opt f l
+
   let rec iter f = function
       Empty -> ()
     | Node(l, v, r, _) -> iter f l; f v; iter f r
@@ -219,9 +327,6 @@ module Concrete = struct
       invalid_arg "Set.at_rank_exn i s: i >= (Set.cardinal s)"
     with Found -> !res
 
-  let map cmp f s =
-    fold (fun v acc -> add cmp (f v) acc) s empty
-
   let rec op_map f = function
     | Empty -> Empty
     | Node (l,x,r,h) -> Node (op_map f l, f x, op_map f r, h)
@@ -230,12 +335,12 @@ module Concrete = struct
 
   let rec add_min v = function
     | Empty -> singleton v
-    | Node (l, x, r, h) ->
+    | Node (l, x, r, _h) ->
       bal (add_min v l) x r
 
   let rec add_max v = function
     | Empty -> singleton v
-    | Node (l, x, r, h) ->
+    | Node (l, x, r, _h) ->
       bal l x (add_max v r)
 
   (* Same as create and bal, but no assumptions are made on the
@@ -331,7 +436,7 @@ module Concrete = struct
 
   let rec cardinal = function
       Empty -> 0
-    | Node(l, v, r, _) -> cardinal l + 1 + cardinal r
+    | Node(l, _v, r, _) -> cardinal l + 1 + cardinal r
 
   let rec elements_aux accu = function
       Empty -> accu
@@ -357,6 +462,14 @@ module Concrete = struct
       Empty -> t
     | Node (l, e, r, _) -> rev_cons_iter r (C (e, l, t))
 
+  let rec cons_iter_from cmp k2 m e =
+    match m with
+    | Empty -> e
+    | Node (l, k, r, _) ->
+       if cmp k2 k <= 0
+       then cons_iter_from cmp k2 l (C (k, r, e))
+       else cons_iter_from cmp k2 r e
+
   let enum_next l () = match !l with
       E -> raise BatEnum.No_more_elements
     | C (e, s, t) -> l := cons_iter s t; e
@@ -368,7 +481,7 @@ module Concrete = struct
   let enum_count l () =
     let rec aux n = function
         E -> n
-      | C (e, s, t) -> aux (n + 1 + cardinal s) t
+      | C (_e, s, t) -> aux (n + 1 + cardinal s) t
     in aux 0 !l
 
   let enum t =
@@ -395,15 +508,14 @@ module Concrete = struct
   let print ?(first="{") ?(last="}") ?(sep=",") print_elt out t =
     BatEnum.print ~first ~last ~sep (fun out e -> BatPrintf.fprintf out "%a" print_elt e) out (enum t)
 
-  let filter cmp f e = fold (fun x acc -> if f x then add cmp x acc else acc) e empty
-
-  let filter_map cmp f e = fold (fun x acc -> match f x with Some v -> add cmp v acc | _ -> acc) e empty
-
   let choose = min_elt (* I'd rather this chose the root, but okay *)
+
   (*$= choose
     42 (empty |> add 42 |> choose)
     (empty |> add 0 |> add 1 |> choose) (empty |> add 1 |> add 0 |> choose)
-  *)
+   *)
+
+  let choose_opt = min_elt_opt
 
   let any = get_root
   (*$T any
@@ -458,6 +570,80 @@ module Concrete = struct
         join (union cmp12 l1 l2) v2 (union cmp12 r1 r2)
       end
 
+  let rec filter p = function
+      Empty -> Empty
+    | (Node(l,v,r,_)) as t ->
+       (* call [p] in the expected left-to-right order *)
+       let l' = filter p l in
+       let pv = p v in
+       let r' = filter p r in
+       if pv then
+         if l==l' && r==r' then t else join l' v r'
+       else concat l' r'
+
+  let try_join cmp l v r =
+    (* [join l v r] can only be called when (elements of l < v <
+         elements of r); use [try_join l v r] when this property may
+         not hold, but you hope it does hold in the common case *)
+    if (l = Empty || cmp (max_elt l) v < 0)
+       && (r = Empty || cmp v (min_elt r) < 0)
+    then join l v r
+    else union cmp l (add cmp v r)
+
+  let rec map_endo cmp f = function
+    | Empty -> Empty
+    | Node(l, v, r, _) as t ->
+       (* enforce left-to-right evaluation order *)
+       let l' = map_endo cmp f l in
+       let v' = f v in
+       let r' = map_endo cmp f r in
+       if l == l' && v == v' && r == r' then t
+       else try_join cmp l' v' r'
+
+  let rec map cmp f = function
+    | Empty -> Empty
+    | Node(l, v, r, _) ->
+       (* enforce left-to-right evaluation order *)
+       let l' = map cmp f l in
+       let v' = f v in
+       let r' = map cmp f r in
+       try_join cmp l' v' r'
+
+  let try_concat cmp t1 t2 =
+    match (t1, t2) with
+      (Empty, t) -> t
+    | (t, Empty) -> t
+    | (_, _) -> try_join cmp t1 (min_elt t2) (remove_min_elt t2)
+
+  let rec filter_map_endo cmp f = function
+    | Empty -> Empty
+    | Node(l, v, r, _) as t ->
+       (* enforce left-to-right evaluation order *)
+       let l' = filter_map_endo cmp f l in
+       let v' = f v in
+       let r' = filter_map_endo cmp f r in
+       begin match v' with
+       | Some v' ->
+          if l == l' && v == v' && r == r' then t
+          else try_join cmp l' v' r'
+       | None ->
+          try_concat cmp l' r'
+       end
+
+  let rec filter_map cmp f = function
+    | Empty -> Empty
+    | Node(l, v, r, _) ->
+       (* enforce left-to-right evaluation order *)
+       let l' = filter_map cmp f l in
+       let v' = f v in
+       let r' = filter_map cmp f r in
+       begin match v' with
+       | Some v' ->
+          try_join cmp l' v' r'
+       | None ->
+          try_concat cmp l' r'
+       end
+
   let rec sym_diff cmp12 s1 s2 =
     match (s1, s2) with
       (Empty, t2) -> t2
@@ -471,8 +657,8 @@ module Concrete = struct
 
   let rec inter cmp12 s1 s2 =
     match (s1, s2) with
-      (Empty, t2) -> Empty
-    | (t1, Empty) -> Empty
+      (Empty, _t2) -> Empty
+    | (_t1, Empty) -> Empty
     | (Node(l1, v1, r1, _), t2) ->
       match split cmp12 v1 t2 with
         (l2, false, r2) ->
@@ -482,7 +668,7 @@ module Concrete = struct
 
   let rec diff cmp12 s1 s2 =
     match (s1, s2) with
-      (Empty, t2) -> Empty
+      (Empty, _t2) -> Empty
     | (t1, Empty) -> t1
     | (Node(l1, v1, r1, _), t2) ->
       match split cmp12 v1 t2 with
@@ -499,7 +685,7 @@ module Concrete = struct
       match split cmp12 v1 t2 with
         (l2, false, r2) ->
         disjoint cmp12 l1 l2 && disjoint cmp12 r1 r2
-      | (l2, true, r2) -> false
+      | (_l2, true, _r2) -> false
 
   let compare cmp s1 s2 =
     let rec compare_aux t1' t2' =
@@ -531,6 +717,34 @@ module Concrete = struct
         subset cmp (Node (l1, v1, Empty, 0)) l2 && subset cmp r1 t2
       else
         subset cmp (Node (Empty, v1, r1, 0)) r2 && subset cmp l1 t2
+
+  let add_seq cmp s m =
+    BatSeq.fold_left (fun m e -> add cmp e m) m s
+
+  let of_seq cmp s =
+    add_seq cmp s empty
+
+  let rec seq_of_iter m () =
+    match m with
+    | E -> BatSeq.Nil
+    | C(k, r, e) ->
+       BatSeq.Cons (k, seq_of_iter (cons_iter r e))
+
+  let to_seq m =
+    seq_of_iter (cons_iter m E)
+
+  let rec rev_seq_of_iter m () =
+    match m with
+    | E -> BatSeq.Nil
+    | C(k, r, e) ->
+       BatSeq.Cons (k, rev_seq_of_iter (rev_cons_iter r e))
+
+  let to_rev_seq m =
+    rev_seq_of_iter (rev_cons_iter m E)
+
+  let to_seq_from cmp k m =
+    seq_of_iter (cons_iter_from cmp k m E)
+
 end
 
 module type S =
@@ -539,11 +753,18 @@ sig
   type t
   val empty: t
   val is_empty: t -> bool
+  val is_singleton: t -> bool
   val singleton: elt -> t
   val mem: elt -> t -> bool
   val find: elt -> t -> elt
+  val find_opt: elt -> t -> elt option
+  val find_first : (elt -> bool) -> t -> elt
+  val find_first_opt : (elt -> bool) -> t -> elt option
+  val find_last : (elt -> bool) -> t -> elt
+  val find_last_opt : (elt -> bool) -> t -> elt option
   val add: elt -> t -> t
   val remove: elt -> t -> t
+  val remove_exn: elt -> t -> t
   val update: elt -> elt -> t -> t
   val union: t -> t -> t
   val inter: t -> t -> t
@@ -572,10 +793,13 @@ sig
   val to_list: t -> elt list
   val to_array: t -> elt array
   val min_elt: t -> elt
+  val min_elt_opt: t -> elt option
   val pop_min: t -> elt * t
   val pop_max: t -> elt * t
   val max_elt: t -> elt
+  val max_elt_opt: t -> elt option
   val choose: t -> elt
+  val choose_opt: t -> elt option
   val any: t -> elt
   val pop: t -> elt * t
   val enum: t -> elt BatEnum.t
@@ -583,9 +807,15 @@ sig
   val of_enum: elt BatEnum.t -> t
   val of_list: elt list -> t
   val of_array: elt array -> t
+  val to_seq : t -> elt BatSeq.t
+  val to_rev_seq : t -> elt BatSeq.t
+  val to_seq_from :  elt -> t -> elt BatSeq.t
+  val add_seq : elt BatSeq.t -> t -> t
+  val of_seq : elt BatSeq.t -> t
   val print :  ?first:string -> ?last:string -> ?sep:string ->
     ('a BatInnerIO.output -> elt -> unit) ->
     'a BatInnerIO.output -> t -> unit
+
   (** Operations on {!Set} without exceptions.*)
   module Exceptionless : sig
     val min_elt: t -> elt option
@@ -594,6 +824,7 @@ sig
     val any:     t -> elt option
     val find: elt -> t -> elt option
   end
+
   (** Operations on {!Set} with labels. *)
   module Labels : sig
     val iter : f:(elt -> unit) -> t -> unit
@@ -611,7 +842,6 @@ end
 module Make (Ord : OrderedType) =
 struct
   include Set.Make(Ord)
-
   (*Breaking the abstraction*)
 
   type implementation = elt Concrete.set
@@ -619,23 +849,31 @@ struct
   external t_of_impl : implementation -> t = "%identity"
 
   let cardinal t = Concrete.cardinal (impl_of_t t)
+  let is_singleton t = Concrete.is_singleton (impl_of_t t)
   let enum t = Concrete.enum (impl_of_t t)
   let of_enum e = t_of_impl (Concrete.of_enum Ord.compare e)
   let backwards t = Concrete.backwards (impl_of_t t)
 
   let remove e t = t_of_impl (Concrete.remove Ord.compare e (impl_of_t t))
+  let remove_exn e t =
+    t_of_impl (Concrete.remove_exn Ord.compare e (impl_of_t t))
   let update e1 e2 t =
     t_of_impl (Concrete.update Ord.compare e1 e2 (impl_of_t t))
   let add e t = t_of_impl (Concrete.add Ord.compare e (impl_of_t t))
 
   let iter f t = Concrete.iter f (impl_of_t t)
   let at_rank_exn i t = Concrete.at_rank_exn i (impl_of_t t)
-  let map f t = t_of_impl (Concrete.map Ord.compare f (impl_of_t t))
+  let map f t = t_of_impl (Concrete.map_endo Ord.compare f (impl_of_t t))
   let fold f t acc = Concrete.fold f (impl_of_t t) acc
-  let filter f t = t_of_impl (Concrete.filter Ord.compare f (impl_of_t t))
-  let filter_map f t = t_of_impl (Concrete.filter_map Ord.compare f (impl_of_t t))
+  let filter f t = t_of_impl (Concrete.filter f (impl_of_t t))
+  let filter_map f t = t_of_impl (Concrete.filter_map_endo Ord.compare f (impl_of_t t))
 
   let find x t = Concrete.find Ord.compare x (impl_of_t t)
+  let find_opt x t = Concrete.find_opt Ord.compare x (impl_of_t t)
+  let find_first f t = Concrete.find_first f (impl_of_t t)
+  let find_first_opt f t = Concrete.find_first_opt f (impl_of_t t)
+  let find_last f t = Concrete.find_last f (impl_of_t t)
+  let find_last_opt f t = Concrete.find_last_opt f (impl_of_t t)
   let exists f t = Concrete.exists f (impl_of_t t)
   let for_all f t = Concrete.for_all f (impl_of_t t)
   let partition f t =
@@ -643,6 +881,7 @@ struct
     (t_of_impl l, t_of_impl r)
 
   let min_elt t = Concrete.min_elt (impl_of_t t)
+  let min_elt_opt t = Concrete.min_elt_opt (impl_of_t t)
   let pop_min t =
     let mini, others = Concrete.pop_min (impl_of_t t) in
     (mini, t_of_impl others)
@@ -651,7 +890,9 @@ struct
     (maxi, t_of_impl others)
 
   let max_elt t = Concrete.max_elt (impl_of_t t)
+  let max_elt_opt t = Concrete.max_elt_opt (impl_of_t t)
   let choose t = Concrete.choose (impl_of_t t)
+  let choose_opt t = Concrete.choose_opt (impl_of_t t)
   let any t = Concrete.any (impl_of_t t)
   let pop t =
     let e, t = Concrete.pop (impl_of_t t) in
@@ -692,11 +933,26 @@ struct
   let subset t1 t2 = Concrete.subset Ord.compare (impl_of_t t1) (impl_of_t t2)
   let disjoint t1 t2 = Concrete.disjoint Ord.compare (impl_of_t t1) (impl_of_t t2)
 
+  let add_seq s t =
+    t_of_impl (Concrete.add_seq Ord.compare s (impl_of_t t))
+
+  let of_seq s =
+    t_of_impl (Concrete.of_seq Ord.compare s)
+
+  let to_seq t =
+    Concrete.to_seq (impl_of_t t)
+
+  let to_rev_seq t =
+    Concrete.to_rev_seq (impl_of_t t)
+
+  let to_seq_from k t =
+    Concrete.to_seq_from Ord.compare k (impl_of_t t)
+
   let rec compare_subset s1 s2 =
     match (s1, impl_of_t s2) with
       (Concrete.Empty, Concrete.Empty) -> 0
-    | (Concrete.Empty, t2) -> -1
-    | (t1, Concrete.Empty) -> 1
+    | (Concrete.Empty, _t2) -> -1
+    | (_t1, Concrete.Empty) -> 1
     | (Concrete.Node(l1, v1, r1, _), t2) ->
       match split v1 (t_of_impl t2) with
         (l2, true, r2) -> (* v1 in both s1 and s2 *)
@@ -785,36 +1041,64 @@ module PSet = struct (*$< PSet *)
 
   let empty    = { cmp = compare; set = Concrete.empty }
   let create cmp  = { cmp = cmp; set = Concrete.empty }
-  let get_cmp {cmp} = cmp
+  let get_cmp {cmp; _} = cmp
 
   (*$T get_cmp
     get_cmp (create BatInt.compare) == BatInt.compare
   *)
 
-
   let singleton ?(cmp = compare) x = { cmp = cmp; set = Concrete.singleton x }
   let is_empty s = Concrete.is_empty s.set
+  let is_singleton s = Concrete.is_singleton s.set
   let mem x s = Concrete.mem s.cmp x s.set
   let find x s = Concrete.find s.cmp x s.set
-  let add x s  = { s with set = Concrete.add s.cmp x s.set }
-  let remove x s = { s with set = Concrete.remove s.cmp x s.set }
-  let update x y s = { s with set = Concrete.update s.cmp x y s.set }
+  let find_opt x s = Concrete.find_opt s.cmp x s.set
+  let find_first f s =  Concrete.find_first f s.set
+  let find_first_opt f s =  Concrete.find_first_opt f s.set
+  let find_last f s =  Concrete.find_last f s.set
+  let find_last_opt f s =  Concrete.find_last_opt f s.set
+  let add x s  =
+    let newset = Concrete.add s.cmp x s.set in
+    if newset == s.set then s
+    else { s with set = newset }
+  let remove x s =
+    let newset = Concrete.remove s.cmp x s.set in
+    if newset == s.set then s
+    else { s with set = newset }
+  let remove_exn x s = { s with set = Concrete.remove_exn s.cmp x s.set }
+  let update x y s =
+    let newset = Concrete.update s.cmp x y s.set in
+    if newset == s.set then s
+    else { s with set =  newset }
   let iter f s = Concrete.iter f s.set
   let at_rank_exn i s = Concrete.at_rank_exn i s.set
   let fold f s acc = Concrete.fold f s.set acc
   let map f s =
     { cmp = Pervasives.compare; set = Concrete.map Pervasives.compare f s.set }
-  let filter f s = { s with set = Concrete.filter s.cmp f s.set }
+  let map_endo f s =
+    let newset = Concrete.map_endo Pervasives.compare f s.set in
+    if s.set == newset then s
+    else { cmp = s.cmp; set = newset }
+  let filter f s =
+    let newset = Concrete.filter f s.set in
+    if newset == s.set then s
+    else { s with set = newset }
   let filter_map f s =
     { cmp = compare; set = Concrete.filter_map compare f s.set }
+  let filter_map_endo f s =
+    let newset = Concrete.filter_map_endo compare f s.set in
+    if newset == s.set then s
+    else { cmp = s.cmp; set = newset }
   let exists f s = Concrete.exists f s.set
   let cardinal s = fold (fun _ acc -> acc + 1) s 0
   let elements s = Concrete.elements s.set
   let to_list = elements
   let to_array s = Concrete.to_array s.set
   let choose s = Concrete.choose s.set
+  let choose_opt s = Concrete.choose_opt s.set
   let any s = Concrete.any s.set
   let min_elt s = Concrete.min_elt s.set
+  let min_elt_opt s = Concrete.min_elt_opt s.set
   let pop_min s =
     let mini, others = Concrete.pop_min s.set in
     (mini, { s with set = others })
@@ -823,6 +1107,7 @@ module PSet = struct (*$< PSet *)
     (maxi, { s with set = others })
 
   let max_elt s = Concrete.max_elt s.set
+  let max_elt_opt s = Concrete.max_elt_opt s.set
   let enum s = Concrete.enum s.set
   let of_enum ?(cmp = compare) e = { cmp; set = Concrete.of_enum compare e }
   let of_enum_cmp ~cmp t = { cmp = cmp; set = Concrete.of_enum cmp t }
@@ -861,6 +1146,23 @@ module PSet = struct (*$< PSet *)
   let equal s1 s2 = Concrete.equal s1.cmp s1.set s2.set
   let subset s1 s2 = Concrete.subset s1.cmp s1.set s2.set
   let disjoint s1 s2 = Concrete.disjoint s1.cmp s1.set s2.set
+
+
+  let add_seq s t =
+    { t with set = Concrete.add_seq t.cmp s t.set }
+
+  let of_seq  ?(cmp = Pervasives.compare) s =
+    {set = Concrete.of_seq cmp s; cmp = cmp }
+
+  let to_seq t =
+    Concrete.to_seq t.set
+
+  let to_rev_seq t =
+    Concrete.to_rev_seq t.set
+
+  let to_seq_from k t =
+    Concrete.to_seq_from t.cmp k t.set
+
 end (*$>*)
 
 type 'a t = 'a Concrete.set
@@ -874,10 +1176,19 @@ let singleton x = Concrete.singleton x
 
 let is_empty s = s = Concrete.Empty
 
+let is_singleton s = Concrete.is_singleton s
+
 let mem x s = Concrete.mem Pervasives.compare x s
 
 let find x s = Concrete.find Pervasives.compare x s
 
+let find_opt x s = Concrete.find_opt Pervasives.compare x s
+
+let find_first f s = Concrete.find_first f s
+let find_last  f s = Concrete.find_last  f s
+let find_first_opt f s = Concrete.find_first_opt f s
+let find_last_opt  f s = Concrete.find_last_opt  f s
+
 (*$T find
   (find 1 (of_list [1;2;3;4;5;6;7;8])) == 1
   (find 8 (of_list [1;2;3;4;5;6;7;8])) == 8
@@ -887,12 +1198,14 @@ let find x s = Concrete.find Pervasives.compare x s
   let x,y = (1,1),(1,1) in find x (singleton y) == y
   let x,y = [|0|],[|0|] in find x (singleton y) != x
   try ignore (find (1,2) (singleton (1,1))); false with Not_found -> true
-*)
+ *)
 
 let add x s  = Concrete.add Pervasives.compare x s
 
 let remove x s = Concrete.remove Pervasives.compare x s
 
+let remove_exn x s = Concrete.remove_exn Pervasives.compare x s
+
 let update x y s = Concrete.update Pervasives.compare x y s
 
 let iter f s = Concrete.iter f s
@@ -912,14 +1225,30 @@ let at_rank_exn i s = Concrete.at_rank_exn i s
 let fold f s acc = Concrete.fold f s acc
 
 let map f s = Concrete.map Pervasives.compare f s
+let map_endo f s = Concrete.map_endo Pervasives.compare f s
 
 (*$T map
   map (fun _x -> 1) (of_list [1;2;3]) |> cardinal = 1
+ *)
+
+(*$T map_endo
+  let s = of_list [1;2;3] in s == (map_endo (fun x -> x) s)
+  let s = empty in s == (map_endo (fun x -> x+1) s)
 *)
 
-let filter f s = Concrete.filter Pervasives.compare f s
+let filter f s = Concrete.filter f s
+(*$T filter
+  let s = of_list [1;2;3] in s == (filter (fun x -> x < 10) s)
+  let s = empty in s == (filter (fun x -> x > 10) s)
+*)
 
 let filter_map f s = Concrete.filter_map Pervasives.compare f s
+let filter_map_endo f s = Concrete.filter_map_endo Pervasives.compare f s
+
+(*$T filter_map_endo
+  let s = of_list [1;2;3] in s == (filter_map_endo (fun x -> Some x) s)
+  let s = empty in s == (filter_map_endo (fun x -> Some x) s)
+*)
 
 let exists f s = Concrete.exists f s
 
@@ -930,9 +1259,18 @@ let to_list = elements
 let to_array s = Concrete.to_array s
 
 let choose s = Concrete.choose s
+let choose_opt s = Concrete.choose_opt s
+
+(*$T choose_opt
+  choose_opt (of_list [1]) = Some 1
+  choose_opt (empty) = None
+  choose_opt (of_list []) = None
+*)
+
 let any s = Concrete.any s
 
 let min_elt s = Concrete.min_elt s
+let min_elt_opt s = Concrete.min_elt_opt s
 
 (*$Q min_elt
   (Q.list Q.small_int) (fun l -> l = [] || \
@@ -948,6 +1286,12 @@ let min_elt s = Concrete.min_elt s
   )
 *)
 
+(*$T min_elt_opt
+  min_elt_opt (of_list [1;2;3]) = Some 1
+  min_elt_opt (empty) = None
+  min_elt_opt (of_list []) = None
+*)
+
 let pop_min s = Concrete.pop_min s
 
 (*$T pop_min
@@ -969,6 +1313,14 @@ let pop_max s = Concrete.pop_max s
 
 let max_elt s = Concrete.max_elt s
 
+let max_elt_opt s = Concrete.max_elt_opt s
+
+(*$T max_elt_opt
+  max_elt_opt (of_list [1;2;3]) = Some 3
+  max_elt_opt (empty) = None
+  max_elt_opt (of_list []) = None
+*)
+
 let enum s = Concrete.enum s
 
 let of_enum e = Concrete.of_enum Pervasives.compare e
@@ -1007,6 +1359,22 @@ let equal s1 s2 = Concrete.equal Pervasives.compare s1 s2
 let subset s1 s2 = Concrete.subset Pervasives.compare s1 s2
 let disjoint s1 s2 = Concrete.disjoint Pervasives.compare s1 s2
 
+
+let add_seq s t =
+  Concrete.add_seq Pervasives.compare s t
+
+let of_seq s =
+  Concrete.of_seq Pervasives.compare s
+
+let to_seq t =
+  Concrete.to_seq t
+
+let to_rev_seq t =
+  Concrete.to_rev_seq t
+
+let to_seq_from k t =
+  Concrete.to_seq_from Pervasives.compare k t
+
 (*$T subset
    subset (of_list [1;2;3]) (of_list [1;2;3;4])
    not (subset (of_list [1;2;3;5]) (of_list [1;2;3;4]))
