@@ -64,6 +64,9 @@ sig
   val is_empty: t -> bool
   (** Test whether a set is empty or not. *)
 
+  val is_singleton: t -> bool
+  (** Test if the set is a singleton. *)
+
   val singleton: elt -> t
   (** [singleton x] returns the one-element set containing only [x]. *)
 
@@ -75,19 +78,73 @@ sig
       @raise Not_found if no element is equal
   *)
 
+  val find_opt : elt -> t -> elt option
+  (** [find_opt x s] returns [Some k] for the element [k] in [s] that
+      tests equal to [x] under its comparison function.
+      If no element is equal, return [None]
+
+      @since 3.3.0 *)
+
+  val find_first : (elt -> bool) -> t -> elt
+  (** [find_first f m] returns the first element [e] for which [f e] is true
+      or raises [Not_found] if there is no such element.
+      [f] must be monotonically increasing,
+      i.e. if [k1 < k2 && f k1] is true then [f k2] must also be true. 
+    
+      @since 3.3.0 *)
+    
+  val find_first_opt : (elt -> bool) -> t -> elt option
+  (** [find_first_opt f m] returns [Some e] for the first element [e]
+      for which [f e] is true or returns [None] if there is no such element.
+      [f] must be monotonically increasing,
+      i.e. if [k1 < k2 && f k1] is true then [f k2] must also be true. 
+    
+      @since 3.3.0 *)
+
+  val find_last : (elt -> bool) -> t -> elt
+  (** [find_last f m] returns the last element [e] for which [f e] is true
+    or raises [Not_found] if there is no such element.
+    [f] must be monotonically decreasing,
+    i.e. if [k1 < k2 && f k2] is true then [f k1] must also be true. 
+    
+    @since 3.3.0 *)
+    
+  val find_last_opt : (elt -> bool) -> t -> elt option
+  (** [find_last_opt f m] returns [Some e] for the last element [e]
+    for which [f e] is true or returns [None] if there is no such element.
+    [f] must be monotonically decreasing,
+    i.e. if [k1 < k2 && f k2] is true then [f k1] must also be true. 
+    
+    @since 3.3.0 *)
+
   val add: elt -> t -> t
   (** [add x s] returns a set containing all elements of [s],
-      plus [x]. If [x] was already in [s], [s] is returned unchanged. *)
+      plus [x]. If [x] was already in [s], [s] is returned unchanged.
+
+      @before 3.3.0 Physical equality was not ensured.
+      *)
 
   val remove: elt -> t -> t
   (** [remove x s] returns a set containing all elements of [s],
-      except [x]. If [x] was not in [s], [s] is returned unchanged. *)
+      except [x]. If [x] was not in [s], [s] is returned unchanged.
+
+      @before 3.3.0 Physical equality was not ensured.
+   *)
+
+  val remove_exn: elt -> t -> t
+  (** [remove_exn x s] behaves like [remove x s] except that it raises
+      an exception if [x] is not in [s].
+      @raise Not_found if [x] is not in [s].
+      @since 3.2.0 *)
 
   val update: elt -> elt -> t -> t
   (** [update x y s] replace [x] by [y] in [s].
       [update] is faster when [x] compares equal to [y] according
       to the comparison function used by your set.
+      When [x] and [y] are physically equal, [m] is returned unchanged.
       @raise Not_found if [x] is not in [s].
+
+      @before 3.3.0 Physical equality was not ensured.
       @since 2.4 *)
 
   val union: t -> t -> t
@@ -139,18 +196,30 @@ sig
   val map: (elt -> elt) -> t -> t
   (** [map f x] creates a new set with elements [f a0],
       [f a1]... [f aN], where [a0],[a1]..[aN] are the
-      values contained in [x]*)
+      values contained in [x]
+
+      if [f] returns all elements unmodified then [x] is returned unmodified.
+      @before 3.3.0 Physical equality was not ensured.
+*)
 
   val filter: (elt -> bool) -> t -> t
   (** [filter p s] returns the set of all elements in [s]
-      that satisfy predicate [p]. *)
+      that satisfy predicate [p]. 
+
+      if [p] returns [true] for all elements then [s] is returned unmodified.
+      @before 3.3.0 Physical equality was not ensured.
+   *)
 
   val filter_map: (elt -> elt option) -> t -> t
   (** [filter_map f m] combines the features of [filter] and
-      [map].  It calls calls [f a0], [f a1], [f aN] where [a0],[a1]..[aN]
-      are the elements of [m] and returns the set of pairs [bi]
+      [map].  It calls [f a0], [f a1], [f aN] where [a0],[a1]..[aN]
+      are the elements of [m] and returns the set of elements [bi]
       such as [f ai = Some bi] (when [f] returns [None], the
-      corresponding element of [m] is discarded). *)
+      corresponding element of [m] is discarded).
+
+      if [f] returns [true] for all elements then [s] is returned unmodified.
+      @before 3.3.0 Physical equality was not ensured.
+ *)
 
   val fold: (elt -> 'a -> 'a) -> t -> 'a -> 'a
   (** [fold f s a] computes [(f xN ... (f x1 (f x0 a))...)],
@@ -225,6 +294,13 @@ sig
 
     @raise Not_found if the set is empty. *)
 
+  val min_elt_opt : t -> elt option
+  (** Return [Some e] for the smallest element [e] of the given set
+      (with respect to the [Ord.compare] ordering).
+      Return None if the set is empty. 
+   
+      @since 3.3.0 *)
+
   val pop_min: t -> elt * t
   (** Returns the smallest element of the given set
       along with the rest of the set.
@@ -248,6 +324,12 @@ sig
   val max_elt: t -> elt
   (** Same as {!Set.S.min_elt}, but returns the largest element of the
       given set. *)
+    
+  val max_elt_opt : t -> elt option
+  (** Same as {!Set.S.min_elt_opt}, but for the largest element of the
+      given set.
+
+      @since 3.3.0 *)
 
   val choose: t -> elt
   (** Return one element of the given set.
@@ -255,6 +337,14 @@ sig
       chosen for equal sets.
       @raise Not_found if the set is empty. *)
 
+  val choose_opt : t -> elt option
+  (** Return [Some e] for one element [e] of the given set.
+      Which element is chosen is unspecified, but equal elements will be
+      chosen for equal sets.
+      Return [None] if the set is empty.
+
+      @since 3.3.0 *)
+
   val any: t -> elt
   (** Return one element of the given set.
       The difference with choose is that there is no guarantee that equals
@@ -290,7 +380,36 @@ sig
   (** builds a set from the given array.
 
       @since 2.4
-  *)
+   *)
+
+
+  val to_seq : t -> elt BatSeq.t
+  (** Iterate on the whole set, in ascending order.
+
+      @since 3.3.0  *)
+
+  val to_rev_seq : t -> elt BatSeq.t
+  (** Iterate on the whole set, in descending order.
+
+      @since 3.3.0  *)
+    
+  val to_seq_from :  elt -> t -> elt BatSeq.t
+  (** [to_seq_from x s] iterates on a subset of the elements in [s], 
+      namely those greater or equal to [x], in ascending order.
+    
+      @since 3.3.0 *)
+    
+  val add_seq : elt BatSeq.t -> t -> t
+  (** add the given elements to the set, in order. 
+    
+      @since 3.3.0  *)
+    
+  val of_seq : elt BatSeq.t -> t
+  (** build a set from the given elements 
+    
+      @since 3.3.0 *)
+     
+
 
 
   (** {6 Boilerplate code}*)
@@ -409,6 +528,9 @@ val empty: 'a t
 val is_empty: 'a t -> bool
 (** Test whether a set is empty or not. *)
 
+val is_singleton: 'a t -> bool
+(** Test if the set is a singleton. *)
+
 val singleton : 'a -> 'a t
 (** Creates a new set with the single given element in it. *)
 
@@ -422,20 +544,75 @@ val find: 'a -> 'a t -> 'a
     @since 2.1
 *)
 
+val find_opt : 'a -> 'a t -> 'a option
+(** [find_opt x s] returns [Some k] for the element [k] in [s] that
+    tests equal to [x] under its comparison function.
+    If no element is equal, return [None]
+
+    @since 3.3.0 *)
+
+val find_first : ('a -> bool) -> 'a t -> 'a
+(** [find_first f m] returns the first element [e] for which [f e] is true
+    or raises [Not_found] if there is no such element.
+    [f] must be monotonically increasing,
+    i.e. if [k1 < k2 && f k1] is true then [f k2] must also be true. 
+  
+    @since 3.3.0 *)
+  
+val find_first_opt : ('a -> bool) -> 'a t -> 'a option
+(** [find_first_opt f m] returns [Some e] for the first element [e]
+    for which [f e] is true or returns [None] if there is no such element.
+    [f] must be monotonically increasing,
+    i.e. if [k1 < k2 && f k1] is true then [f k2] must also be true. 
+  
+    @since 3.3.0 *)
+
+val find_last : ('a -> bool) -> 'a t -> 'a
+(** [find_last f m] returns the last element [e] for which [f e] is true
+  or raises [Not_found] if there is no such element.
+  [f] must be monotonically decreasing,
+  i.e. if [k1 < k2 && f k2] is true then [f k1] must also be true. 
+  
+  @since 3.3.0 *)
+  
+val find_last_opt : ('a -> bool) -> 'a t -> 'a option
+(** [find_last_opt f m] returns [Some e] for the last element [e]
+  for which [f e] is true or returns [None] if there is no such element.
+  [f] must be monotonically decreasing,
+  i.e. if [k1 < k2 && f k2] is true then [f k1] must also be true. 
+  
+  @since 3.3.0 *)
+
 val add: 'a -> 'a t -> 'a t
 (** [add x s] returns a set containing all elements of [s],
-    plus [x]. If [x] was already in [s], [s] is returned unchanged. *)
+    plus [x]. If [x] was already in [s], [s] is returned unchanged.
+
+    @before 3.3.0 Physical equality was not ensured.
+ *)
 
 val remove: 'a -> 'a t -> 'a t
 (** [remove x s] returns a set containing all elements of [s],
-    except [x]. If [x] was not in [s], [s] is returned unchanged. *)
+    except [x]. If [x] was not in [s], [s] is returned unchanged. 
+
+    @before 3.3.0 Physical equality was not ensured.
+*)
+
+val remove_exn: 'a -> 'a t -> 'a t
+(** [remove_exn x s] behaves like [remove x s] except that it raises
+    an exception if [x] is not in [s].
+    @raise Not_found if [x] is not in [s].
+    @since 3.2.0 *)
 
 val update: 'a -> 'a -> 'a t -> 'a t
 (** [update x y s] replace [x] by [y] in [s].
     [update] is faster when [x] compares equal to [y] according
     to the comparison function used by your set.
+    When [x] and [y] are physically equal, [m] is returned unchanged.
     @raise Not_found if [x] is not in [s].
-    @since 2.4 *)
+    @since 2.4 
+
+    @before 3.3.0 Physical equality was not ensured.
+*)
 
 val union: 'a t -> 'a t -> 'a t
 (** [union s t] returns the union of [s] and [t] - the set containing
@@ -498,9 +675,29 @@ val map: ('a -> 'b) -> 'a t -> 'b t
     [Incubator.op_map] may be more efficient.
 *)
 
+val map_endo: ('a -> 'a) -> 'a t -> 'a t
+(** [map_endo f x] creates a new set with elements [f a0],
+    [f a1]... [f aN], where [a0], [a1], ..., [aN] are the
+    elements of [x].
+
+    This function places no restriction on [f] (beyond the type 
+    signature being more restricted than for [map] above); it can
+    map multiple input values to the same output value, in which
+    case the resulting set will have smaller cardinality than the
+    input.  [f]  does not need to be order preserving, although if
+    it is, then [Incubator.op_map] may be more efficient.
+
+    This version of map will result in a physically equal map if [f]
+    returns physically equal keys.
+*)
+
 val filter: ('a -> bool) -> 'a t -> 'a t
 (** [filter p s] returns the set of all elements in [s]
-    that satisfy predicate [p]. *)
+    that satisfy predicate [p]. 
+
+    if [p] returns [true] for all elements then [s] is returned unmodified.
+    @before 3.3.0 Physical equality was not ensured.
+ *)
 
 (* as under-specified as 'map' *)
 val filter_map: ('a -> 'b option) -> 'a t -> 'b t
@@ -514,6 +711,19 @@ val filter_map: ('a -> 'b option) -> 'a t -> 'b t
     order elements.
 *)
 
+val filter_map_endo: ('a -> 'a option) -> 'a t -> 'a t
+(** [filter_map_endo f m] combines the features of [filter] and
+    [map].  It calls calls [f a0], [f a1], [f aN] where [a0,a1..an]
+    are the elements of [m] and returns the set of pairs [bi]
+    such as [f ai = Some bi] (when [f] returns [None], the
+    corresponding element of [m] is discarded).
+
+    The resulting map uses the polymorphic [compare] function to
+    order elements.
+
+    If the filter function [f] returns [true] for all elements in [m],
+    the resulting map is physically equal to [m]. *)
+
 val fold: ('a -> 'b -> 'b) -> 'a t -> 'b -> 'b
 (** [fold f s a] computes [(f xN ... (f x1 (f x0 a))...)],
     where [x0,x1..xN] are the elements of [s], in increasing order. *)
@@ -581,6 +791,13 @@ val min_elt : 'a t -> 'a
 (** returns the smallest element of the set.
     @raise Not_found if given an empty set. *)
 
+val min_elt_opt : 'a t -> 'a option
+(** Return [Some e] for the smallest element [e] of the given set
+    (with respect to the [Ord.compare] ordering).
+    Return None if the set is empty. 
+ 
+    @since 3.3.0 *)
+
 val pop_min: 'a t -> 'a * 'a t
 (** Returns the smallest element of the given set
     along with the rest of the set.
@@ -605,10 +822,24 @@ val max_elt : 'a t -> 'a
 (** returns the largest element of the set.
     @raise Not_found if given an empty set.*)
 
+val max_elt_opt : 'a t -> 'a option
+(** Same as {!Set.S.min_elt_opt}, but for the largest element of the
+    given set.
+
+    @since 3.3.0 *)
+
 val choose : 'a t -> 'a
 (** returns an arbitrary (but deterministic) element of the given set.
     @raise Not_found if given an empty set. *)
 
+val choose_opt : 'a t -> 'a option
+(** Return [Some e] for one element [e] of the given set.
+    Which element is chosen is unspecified, but equal elements will be
+    chosen for equal sets.
+    Return [None] if the set is empty.
+
+    @since 3.3.0 *)
+
 val any: 'a t -> 'a
 (** Return one element of the given set.
     The difference with choose is that there is no guarantee that equals
@@ -645,6 +876,33 @@ val of_array: 'a array -> 'a t
 (** builds a set from the given array, using the default comparison
     function *)
 
+  
+val to_seq : 'a t -> 'a BatSeq.t
+(** Iterate on the whole set, in ascending order.
+
+    @since 3.3.0  *)
+  
+val to_rev_seq : 'a t -> 'a BatSeq.t
+(** Iterate on the whole set, in descending order.
+
+    @since 3.3.0  *)
+  
+val to_seq_from :  'a -> 'a t -> 'a BatSeq.t
+(** [to_seq_from x s] iterates on a subset of the elements in [s], 
+    namely those greater or equal to [x], in ascending order.
+  
+    @since 3.3.0 *)
+  
+val add_seq : 'a BatSeq.t -> 'a t -> 'a t
+(** add the given elements to the set, in order. 
+  
+    @since 3.3.0  *)
+  
+val of_seq : 'a BatSeq.t -> 'a t
+(** build a set from the given elements 
+  
+    @since 3.3.0 *)
+   
 (** {6 Boilerplate code}*)
 
 
@@ -702,6 +960,9 @@ module PSet : sig
   val is_empty: 'a t -> bool
   (** Test whether a set is empty or not. *)
 
+  val is_singleton: 'a t -> bool
+  (** Test if the set is a singleton. *)
+
   val singleton : ?cmp:('a -> 'a -> int) -> 'a -> 'a t
   (** Creates a new set with the single given element in it. *)
 
@@ -713,19 +974,72 @@ module PSet : sig
       @raise Not_found if no element is equal
   *)
 
+  val find_opt : 'a -> 'a t -> 'a option
+  (** [find_opt x s] returns [Some k] for the element [k] in [s] that
+      tests equal to [x] under its comparison function.
+      If no element is equal, return [None]
+
+      @since 3.3.0 *)
+
+  val find_first : ('a -> bool) -> 'a t -> 'a
+  (** [find_first f m] returns the first element [e] for which [f e] is true
+      or raises [Not_found] if there is no such element.
+      [f] must be monotonically increasing,
+      i.e. if [k1 < k2 && f k1] is true then [f k2] must also be true. 
+    
+      @since 3.3.0 *)
+    
+  val find_first_opt : ('a -> bool) -> 'a t -> 'a option
+  (** [find_first_opt f m] returns [Some e] for the first element [e]
+      for which [f e] is true or returns [None] if there is no such element.
+      [f] must be monotonically increasing,
+      i.e. if [k1 < k2 && f k1] is true then [f k2] must also be true. 
+    
+      @since 3.3.0 *)
+
+  val find_last : ('a -> bool) -> 'a t -> 'a
+  (** [find_last f m] returns the last element [e] for which [f e] is true
+    or raises [Not_found] if there is no such element.
+    [f] must be monotonically decreasing,
+    i.e. if [k1 < k2 && f k2] is true then [f k1] must also be true. 
+    
+    @since 3.3.0 *)
+    
+  val find_last_opt : ('a -> bool) -> 'a t -> 'a option
+  (** [find_last_opt f m] returns [Some e] for the last element [e]
+    for which [f e] is true or returns [None] if there is no such element.
+    [f] must be monotonically decreasing,
+    i.e. if [k1 < k2 && f k2] is true then [f k1] must also be true. 
+    
+    @since 3.3.0 *)
+
   val add: 'a -> 'a t -> 'a t
   (** [add x s] returns a set containing all elements of [s],
-      plus [x]. If [x] was already in [s], [s] is returned unchanged. *)
+      plus [x]. If [x] was already in [s], [s] is returned unchanged. 
+
+      @before 3.3.0 Physical equality was not ensured.
+   *)
 
   val remove: 'a -> 'a t -> 'a t
   (** [remove x s] returns a set containing all elements of [s],
-      except [x]. If [x] was not in [s], [s] is returned unchanged. *)
+      except [x]. If [x] was not in [s], [s] is returned unchanged.
+
+      @before 3.3.0 Physical equality was not ensured.
+   *)
+
+  val remove_exn: 'a -> 'a t -> 'a t
+  (** [remove_exn x s] behaves like [remove x s] except that it raises
+      an exception if [x] is not in [s].
+      @raise Not_found if [x] is not in [s].
+      @since 3.2.0 *)
 
   val update: 'a -> 'a -> 'a t -> 'a t
   (** [update x y s] replace [x] by [y] in [s].
       [update] is faster when [x] compares equal to [y] according
       to the comparison function used by your set.
+      When [x] and [y] are physically equal, [m] is returned unchanged.
       @raise Not_found if [x] is not in [s].
+      @before 3.3.0 Physical equality was not ensured.
       @since 2.4 *)
 
   val union: 'a t -> 'a t -> 'a t
@@ -787,9 +1101,26 @@ module PSet : sig
       order elements.
   *)
 
+  val map_endo: ('a -> 'a) -> 'a t -> 'a t
+  (** [map f x] creates a new set with elements [f a0],
+      [f a1]... [f aN], where [a0], [a1], ..., [aN] are the
+      values contained in [x]
+
+      The resulting map uses the same [compare] function to
+      order elements as [m] does.
+
+      If [f] returns physically equal values for all elements 
+      in [m] then the resulting map will be physically equal to [m].
+      @since 3.3.0
+  *)
+
   val filter: ('a -> bool) -> 'a t -> 'a t
   (** [filter p s] returns the set of all elements in [s]
-      that satisfy predicate [p]. *)
+      that satisfy predicate [p]. 
+
+      if [p] returns [true] for all elements then [s] is returned unmodified.
+      @before 3.3.0 Physical equality was not ensured.
+   *)
 
   (* as under-specified as 'map' *)
   val filter_map: ('a -> 'b option) -> 'a t -> 'b t
@@ -803,6 +1134,21 @@ module PSet : sig
       order elements.
   *)
 
+  val filter_map_endo: ('a -> 'a option) -> 'a t -> 'a t
+  (** [filter_map_endo f m] combines the features of [filter] and
+      [map].  It calls calls [f a0], [f a1], [f aN] where [a0,a1..an]
+      are the elements of [m] and returns the set of pairs [bi]
+      such as [f ai = Some bi] (when [f] returns [None], the
+      corresponding element of [m] is discarded).
+  
+      The resulting map uses the same [compare] function to
+      order elements as used for [m].
+  
+      if the filter function [f] returns [true] for all elements in [m],
+      the resulting map is physically equal to [m].
+      @since 3.3.0
+  *)
+
   val fold: ('a -> 'b -> 'b) -> 'a t -> 'b -> 'b
   (** [fold f s a] computes [(f xN ... (f x1 (f x0 a))...)],
       where [x0,x1..xN] are the elements of [s], in increasing order. *)
@@ -866,6 +1212,13 @@ module PSet : sig
   (** returns the smallest element of the set.
       @raise Not_found if given an empty set. *)
 
+  val min_elt_opt : 'a t -> 'a option
+  (** Return [Some e] for the smallest element [e] of the given set
+      (with respect to the [Ord.compare] ordering).
+      Return None if the set is empty. 
+   
+      @since 3.3.0 *)
+
   val pop_min: 'a t -> 'a * 'a t
   (** Returns the smallest element of the given set
       along with the rest of the set.
@@ -890,10 +1243,24 @@ module PSet : sig
   (** returns the largest element of the set.
       @raise Not_found if given an empty set.*)
 
+  val max_elt_opt : 'a t -> 'a option
+  (** Same as {!Set.S.min_elt_opt}, but for the largest element of the
+      given set.
+  
+      @since 3.3.0 *)
+  
   val choose : 'a t -> 'a
   (** returns an arbitrary (but deterministic) element of the given set.
       @raise Not_found if given an empty set. *)
 
+  val choose_opt : 'a t -> 'a option
+  (** Return [Some e] for one element [e] of the given set.
+      Which element is chosen is unspecified, but equal elements will be
+      chosen for equal sets.
+      Return [None] if the set is empty.
+  
+      @since 3.3.0 *)
+  
   val any: 'a t -> 'a
   (** Return one element of the given set.
       The difference with choose is that there is no guarantee that equals
@@ -922,6 +1289,33 @@ module PSet : sig
   val of_array: ?cmp:('a -> 'a -> int) -> 'a array -> 'a t
   (** builds a set from the given array and comparison function *)
 
+      
+  val to_seq : 'a t -> 'a BatSeq.t
+  (** Iterate on the whole set, in ascending order.
+  
+      @since 3.3.0  *)
+    
+  val to_rev_seq : 'a t -> 'a BatSeq.t
+  (** Iterate on the whole set, in descending order.
+  
+      @since 3.3.0  *)
+    
+  val to_seq_from :  'a -> 'a t -> 'a BatSeq.t
+  (** [to_seq_from x s] iterates on a subset of the elements in [s], 
+      namely those greater or equal to [x], in ascending order.
+    
+      @since 3.3.0 *)
+    
+  val add_seq : 'a BatSeq.t -> 'a t -> 'a t
+  (** add the given elements to the set, in order. 
+    
+      @since 3.3.0  *)
+    
+  val of_seq : ?cmp:('a -> 'a -> int) -> 'a BatSeq.t -> 'a t
+  (** build a set from the given elements 
+    
+      @since 3.3.0 *)
+     
   (** {6 Boilerplate code}*)
 
 
